@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Interface to Weka Classsifiers
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2017 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -15,9 +15,10 @@ import os
 import subprocess
 import re
 import zipfile
-
 from sys import stdin
-from nltk import compat
+
+from six import integer_types, string_types
+
 from nltk.probability import DictionaryProbDist
 from nltk.internals import java, config_java
 
@@ -265,9 +266,9 @@ class ARFF_Formatter:
             for (fname, fval) in tok.items():
                 if issubclass(type(fval), bool):
                     ftype = '{True, False}'
-                elif issubclass(type(fval), (compat.integer_types, float, bool)):
+                elif issubclass(type(fval), (integer_types, float, bool)):
                     ftype = 'NUMERIC'
-                elif issubclass(type(fval), compat.string_types):
+                elif issubclass(type(fval), string_types):
                     ftype = 'STRING'
                 elif fval is None:
                     continue # can't tell the type.
@@ -329,7 +330,7 @@ class ARFF_Formatter:
     def _fmt_arff_val(self, fval):
         if fval is None:
             return '?'
-        elif isinstance(fval, (bool, compat.integer_types)):
+        elif isinstance(fval, (bool, integer_types)):
             return '%s' % fval
         elif isinstance(fval, float):
             return '%r' % fval
